import _extends from "@babel/runtime/helpers/extends";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useState, useRef } from 'react';
import Form from '../form';
import arrayMutators from 'final-form-arrays';
import PropTypes from 'prop-types';
import createFocusDecorator from 'final-form-focus';
import RendererContext from '../renderer-context';
import renderForm from './render-form';
import defaultSchemaValidator from '../default-schema-validator';
import SchemaErrorComponent from './schema-error-component';
import defaultValidatorMapper from '../validator-mapper';

var FormRenderer = function FormRenderer(_ref) {
  var componentMapper = _ref.componentMapper,
      FormTemplate = _ref.FormTemplate,
      _onSubmit = _ref.onSubmit,
      onCancel = _ref.onCancel,
      _onReset = _ref.onReset,
      clearOnUnmount = _ref.clearOnUnmount,
      subscription = _ref.subscription,
      clearedValue = _ref.clearedValue,
      schema = _ref.schema,
      validatorMapper = _ref.validatorMapper,
      actionMapper = _ref.actionMapper,
      schemaValidatorMapper = _ref.schemaValidatorMapper,
      props = _objectWithoutProperties(_ref, ["componentMapper", "FormTemplate", "onSubmit", "onCancel", "onReset", "clearOnUnmount", "subscription", "clearedValue", "schema", "validatorMapper", "actionMapper", "schemaValidatorMapper"]);

  var _useState = useState([]),
      _useState2 = _slicedToArray(_useState, 2),
      fileInputs = _useState2[0],
      setFileInputs = _useState2[1];

  var focusDecorator = useRef(createFocusDecorator());
  var schemaError;

  var validatorMapperMerged = _objectSpread(_objectSpread({}, defaultValidatorMapper), validatorMapper);

  try {
    var validatorTypes = Object.keys(validatorMapperMerged);
    var actionTypes = actionMapper ? Object.keys(actionMapper) : [];
    defaultSchemaValidator(schema, componentMapper, validatorTypes, actionTypes, schemaValidatorMapper);
  } catch (error) {
    schemaError = error;
    console.error(error);
    console.log('error: ', error.message);
  }

  if (schemaError) {
    return /*#__PURE__*/React.createElement(SchemaErrorComponent, {
      name: schemaError.name,
      message: schemaError.message
    });
  }

  var registerInputFile = function registerInputFile(name) {
    return setFileInputs(function (prevFiles) {
      return [].concat(_toConsumableArray(prevFiles), [name]);
    });
  };

  var unRegisterInputFile = function unRegisterInputFile(name) {
    return setFileInputs(function (prevFiles) {
      return _toConsumableArray(prevFiles.splice(prevFiles.indexOf(name)));
    });
  };

  return /*#__PURE__*/React.createElement(Form, _extends({}, props, {
    onSubmit: function onSubmit(values, formApi) {
      for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        args[_key - 2] = arguments[_key];
      }

      return _onSubmit.apply(void 0, [values, _objectSpread(_objectSpread({}, formApi), {}, {
        fileInputs: fileInputs
      })].concat(args));
    },
    mutators: _objectSpread({}, arrayMutators),
    decorators: [focusDecorator.current],
    subscription: _objectSpread({
      pristine: true,
      submitting: true,
      valid: true
    }, subscription),
    render: function render(_ref2) {
      var handleSubmit = _ref2.handleSubmit,
          pristine = _ref2.pristine,
          valid = _ref2.valid,
          _ref2$form = _ref2.form,
          reset = _ref2$form.reset,
          mutators = _ref2$form.mutators,
          getState = _ref2$form.getState,
          submit = _ref2$form.submit,
          form = _objectWithoutProperties(_ref2$form, ["reset", "mutators", "getState", "submit"]);

      return /*#__PURE__*/React.createElement(RendererContext.Provider, {
        value: {
          componentMapper: componentMapper,
          validatorMapper: validatorMapperMerged,
          actionMapper: actionMapper,
          formOptions: _objectSpread(_objectSpread({
            registerInputFile: registerInputFile,
            unRegisterInputFile: unRegisterInputFile,
            pristine: pristine,
            onSubmit: _onSubmit,
            onCancel: onCancel ? function () {
              for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = arguments[_key2];
              }

              return onCancel.apply(void 0, [getState().values].concat(args));
            } : undefined,
            onReset: function onReset() {
              _onReset && _onReset.apply(void 0, arguments);
              reset();
            },
            getState: getState,
            valid: valid,
            clearedValue: clearedValue,
            submit: submit,
            handleSubmit: handleSubmit,
            reset: reset,
            clearOnUnmount: clearOnUnmount,
            renderForm: renderForm
          }, mutators), form)
        }
      }, /*#__PURE__*/React.createElement(FormTemplate, {
        formFields: renderForm(schema.fields),
        schema: schema
      }));
    }
  }));
};

FormRenderer.propTypes = {
  onSubmit: PropTypes.func.isRequired,
  onCancel: PropTypes.func,
  onReset: PropTypes.func,
  schema: PropTypes.object.isRequired,
  clearOnUnmount: PropTypes.bool,
  subscription: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.bool)),
  clearedValue: PropTypes.any,
  componentMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.oneOfType([PropTypes.node, PropTypes.element, PropTypes.func]))).isRequired,
  FormTemplate: PropTypes.func.isRequired,
  validatorMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
  actionMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
  schemaValidatorMapper: PropTypes.shape({
    components: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
    validators: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
    actions: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func))
  })
};
FormRenderer.defaultProps = {
  initialValues: {},
  clearOnUnmount: false
};
export default FormRenderer;