import _extends from "@babel/runtime/helpers/extends";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _toArray from "@babel/runtime/helpers/toArray";
import _typeof from "@babel/runtime/helpers/typeof";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useContext } from 'react';
import PropTypes from 'prop-types';
import { childrenPropTypes } from '@data-driven-forms/common/prop-types-templates';
import RendererContext from '../renderer-context';
import Condition from '../condition';
import FormSpy from '../form-spy';

var FormFieldHideWrapper = function FormFieldHideWrapper(_ref) {
  var hideField = _ref.hideField,
      children = _ref.children;
  return hideField ? /*#__PURE__*/React.createElement("div", {
    hidden: true
  }, children) : children;
};

FormFieldHideWrapper.propTypes = {
  hideField: PropTypes.bool,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]).isRequired
};
FormFieldHideWrapper.defaultProps = {
  hideField: false
};

var FormConditionWrapper = function FormConditionWrapper(_ref2) {
  var condition = _ref2.condition,
      children = _ref2.children,
      field = _ref2.field;
  return condition ? /*#__PURE__*/React.createElement(FormSpy, {
    subscription: {
      values: true
    }
  }, function (_ref3) {
    var values = _ref3.values;
    return /*#__PURE__*/React.createElement(Condition, {
      condition: condition,
      values: values,
      field: field
    }, children);
  }) : children;
};

FormConditionWrapper.propTypes = {
  condition: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
  children: childrenPropTypes.isRequired,
  field: PropTypes.object
};

var SingleField = function SingleField(_ref4) {
  var component = _ref4.component,
      condition = _ref4.condition,
      hideField = _ref4.hideField,
      rest = _objectWithoutProperties(_ref4, ["component", "condition", "hideField"]);

  var _useContext = useContext(RendererContext),
      actionMapper = _useContext.actionMapper,
      componentMapper = _useContext.componentMapper;

  var componentProps = _objectSpread({
    component: component
  }, rest);

  var componentBinding = componentMapper[component];
  var Component;

  if (_typeof(componentBinding) === 'object' && Object.prototype.hasOwnProperty.call(componentBinding, 'component')) {
    var _component = componentBinding.component,
        mapperProps = _objectWithoutProperties(componentBinding, ["component"]);

    Component = _component;
    componentProps = _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, mapperProps), componentProps), mapperProps.actions && rest.actions ? {
      actions: _objectSpread(_objectSpread({}, mapperProps.actions), rest.actions)
    } : {}), mapperProps.resolveProps && rest.resolveProps ? {
      resolveProps: function resolveProps() {
        return _objectSpread(_objectSpread({}, mapperProps.resolveProps.apply(mapperProps, arguments)), rest.resolveProps.apply(rest, arguments));
      }
    } : {});
  } else {
    Component = componentBinding;
  }
  /**
   * Map actions to props
   */


  var overrideProps = {};
  var mergedResolveProps; // new object has to be created because of references

  if (componentProps.actions) {
    Object.keys(componentProps.actions).forEach(function (prop) {
      var _componentProps$actio = _toArray(componentProps.actions[prop]),
          action = _componentProps$actio[0],
          args = _componentProps$actio.slice(1);

      overrideProps[prop] = actionMapper[action].apply(actionMapper, _toConsumableArray(args));
    }); // Merge componentProps resolve props and actions resolve props

    if (componentProps.resolveProps && overrideProps.resolveProps) {
      mergedResolveProps = function mergedResolveProps() {
        var _componentProps;

        return _objectSpread(_objectSpread({}, (_componentProps = componentProps).resolveProps.apply(_componentProps, arguments)), overrideProps.resolveProps.apply(overrideProps, arguments));
      };
    } // do not pass actions object to components


    delete componentProps.actions;
  }

  return /*#__PURE__*/React.createElement(FormConditionWrapper, {
    condition: condition,
    field: componentProps
  }, /*#__PURE__*/React.createElement(FormFieldHideWrapper, {
    hideField: hideField
  }, /*#__PURE__*/React.createElement(Component, _extends({}, componentProps, overrideProps, mergedResolveProps && {
    resolveProps: mergedResolveProps
  }))));
};

SingleField.propTypes = {
  component: PropTypes.string.isRequired,
  condition: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
  hideField: PropTypes.bool,
  dataType: PropTypes.string,
  validate: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.func, PropTypes.object])),
  initialValue: PropTypes.any,
  actions: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
  resolveProps: PropTypes.func
};

var renderForm = function renderForm(fields) {
  return fields.map(function (field) {
    return Array.isArray(field) ? renderForm(field) : /*#__PURE__*/React.createElement(SingleField, _extends({
      key: field.name
    }, field));
  });
};

export default renderForm;